# 1 "../../c/uwapi-c/uwapi/bots.h"
# 1 "<built-in>" 1
# 1 "<built-in>" 3
# 385 "<built-in>" 3
# 1 "<command line>" 1
# 1 "<built-in>" 2
# 1 "../../c/uwapi-c/uwapi/bots.h" 2



# 1 "../../c/uwapi-c/uwapi/common.h" 1



# 1 "/usr/bin/../lib/clang/18/include/stdbool.h" 1 3
# 5 "../../c/uwapi-c/uwapi/common.h" 2
# 1 "/usr/bin/../lib/clang/18/include/stdint.h" 1 3
# 52 "/usr/bin/../lib/clang/18/include/stdint.h" 3
# 1 "/usr/include/stdint.h" 1 3 4
# 26 "/usr/include/stdint.h" 3 4
# 1 "/usr/include/bits/libc-header-start.h" 1 3 4
# 33 "/usr/include/bits/libc-header-start.h" 3 4
# 1 "/usr/include/features.h" 1 3 4
# 394 "/usr/include/features.h" 3 4
# 1 "/usr/include/features-time64.h" 1 3 4
# 20 "/usr/include/features-time64.h" 3 4
# 1 "/usr/include/bits/wordsize.h" 1 3 4
# 21 "/usr/include/features-time64.h" 2 3 4
# 1 "/usr/include/bits/timesize.h" 1 3 4
# 19 "/usr/include/bits/timesize.h" 3 4
# 1 "/usr/include/bits/wordsize.h" 1 3 4
# 20 "/usr/include/bits/timesize.h" 2 3 4
# 22 "/usr/include/features-time64.h" 2 3 4
# 395 "/usr/include/features.h" 2 3 4
# 481 "/usr/include/features.h" 3 4
# 1 "/usr/include/stdc-predef.h" 1 3 4
# 482 "/usr/include/features.h" 2 3 4
# 503 "/usr/include/features.h" 3 4
# 1 "/usr/include/sys/cdefs.h" 1 3 4
# 576 "/usr/include/sys/cdefs.h" 3 4
# 1 "/usr/include/bits/wordsize.h" 1 3 4
# 577 "/usr/include/sys/cdefs.h" 2 3 4
# 1 "/usr/include/bits/long-double.h" 1 3 4
# 578 "/usr/include/sys/cdefs.h" 2 3 4
# 504 "/usr/include/features.h" 2 3 4
# 527 "/usr/include/features.h" 3 4
# 1 "/usr/include/gnu/stubs.h" 1 3 4
# 10 "/usr/include/gnu/stubs.h" 3 4
# 1 "/usr/include/gnu/stubs-64.h" 1 3 4
# 11 "/usr/include/gnu/stubs.h" 2 3 4
# 528 "/usr/include/features.h" 2 3 4
# 34 "/usr/include/bits/libc-header-start.h" 2 3 4
# 27 "/usr/include/stdint.h" 2 3 4
# 1 "/usr/include/bits/types.h" 1 3 4
# 27 "/usr/include/bits/types.h" 3 4
# 1 "/usr/include/bits/wordsize.h" 1 3 4
# 28 "/usr/include/bits/types.h" 2 3 4
# 1 "/usr/include/bits/timesize.h" 1 3 4
# 19 "/usr/include/bits/timesize.h" 3 4
# 1 "/usr/include/bits/wordsize.h" 1 3 4
# 20 "/usr/include/bits/timesize.h" 2 3 4
# 29 "/usr/include/bits/types.h" 2 3 4


typedef unsigned char __u_char;
typedef unsigned short int __u_short;
typedef unsigned int __u_int;
typedef unsigned long int __u_long;


typedef signed char __int8_t;
typedef unsigned char __uint8_t;
typedef signed short int __int16_t;
typedef unsigned short int __uint16_t;
typedef signed int __int32_t;
typedef unsigned int __uint32_t;

typedef signed long int __int64_t;
typedef unsigned long int __uint64_t;






typedef __int8_t __int_least8_t;
typedef __uint8_t __uint_least8_t;
typedef __int16_t __int_least16_t;
typedef __uint16_t __uint_least16_t;
typedef __int32_t __int_least32_t;
typedef __uint32_t __uint_least32_t;
typedef __int64_t __int_least64_t;
typedef __uint64_t __uint_least64_t;



typedef long int __quad_t;
typedef unsigned long int __u_quad_t;







typedef long int __intmax_t;
typedef unsigned long int __uintmax_t;
# 141 "/usr/include/bits/types.h" 3 4
# 1 "/usr/include/bits/typesizes.h" 1 3 4
# 142 "/usr/include/bits/types.h" 2 3 4
# 1 "/usr/include/bits/time64.h" 1 3 4
# 143 "/usr/include/bits/types.h" 2 3 4


typedef unsigned long int __dev_t;
typedef unsigned int __uid_t;
typedef unsigned int __gid_t;
typedef unsigned long int __ino_t;
typedef unsigned long int __ino64_t;
typedef unsigned int __mode_t;
typedef unsigned long int __nlink_t;
typedef long int __off_t;
typedef long int __off64_t;
typedef int __pid_t;
typedef struct { int __val[2]; } __fsid_t;
typedef long int __clock_t;
typedef unsigned long int __rlim_t;
typedef unsigned long int __rlim64_t;
typedef unsigned int __id_t;
typedef long int __time_t;
typedef unsigned int __useconds_t;
typedef long int __suseconds_t;
typedef long int __suseconds64_t;

typedef int __daddr_t;
typedef int __key_t;


typedef int __clockid_t;


typedef void * __timer_t;


typedef long int __blksize_t;




typedef long int __blkcnt_t;
typedef long int __blkcnt64_t;


typedef unsigned long int __fsblkcnt_t;
typedef unsigned long int __fsblkcnt64_t;


typedef unsigned long int __fsfilcnt_t;
typedef unsigned long int __fsfilcnt64_t;


typedef long int __fsword_t;

typedef long int __ssize_t;


typedef long int __syscall_slong_t;

typedef unsigned long int __syscall_ulong_t;



typedef __off64_t __loff_t;
typedef char *__caddr_t;


typedef long int __intptr_t;


typedef unsigned int __socklen_t;




typedef int __sig_atomic_t;
# 28 "/usr/include/stdint.h" 2 3 4
# 1 "/usr/include/bits/wchar.h" 1 3 4
# 29 "/usr/include/stdint.h" 2 3 4
# 1 "/usr/include/bits/wordsize.h" 1 3 4
# 30 "/usr/include/stdint.h" 2 3 4




# 1 "/usr/include/bits/stdint-intn.h" 1 3 4
# 24 "/usr/include/bits/stdint-intn.h" 3 4
typedef __int8_t int8_t;
typedef __int16_t int16_t;
typedef __int32_t int32_t;
typedef __int64_t int64_t;
# 35 "/usr/include/stdint.h" 2 3 4


# 1 "/usr/include/bits/stdint-uintn.h" 1 3 4
# 24 "/usr/include/bits/stdint-uintn.h" 3 4
typedef __uint8_t uint8_t;
typedef __uint16_t uint16_t;
typedef __uint32_t uint32_t;
typedef __uint64_t uint64_t;
# 38 "/usr/include/stdint.h" 2 3 4



# 1 "/usr/include/bits/stdint-least.h" 1 3 4
# 25 "/usr/include/bits/stdint-least.h" 3 4
typedef __int_least8_t int_least8_t;
typedef __int_least16_t int_least16_t;
typedef __int_least32_t int_least32_t;
typedef __int_least64_t int_least64_t;


typedef __uint_least8_t uint_least8_t;
typedef __uint_least16_t uint_least16_t;
typedef __uint_least32_t uint_least32_t;
typedef __uint_least64_t uint_least64_t;
# 42 "/usr/include/stdint.h" 2 3 4





typedef signed char int_fast8_t;

typedef long int int_fast16_t;
typedef long int int_fast32_t;
typedef long int int_fast64_t;
# 60 "/usr/include/stdint.h" 3 4
typedef unsigned char uint_fast8_t;

typedef unsigned long int uint_fast16_t;
typedef unsigned long int uint_fast32_t;
typedef unsigned long int uint_fast64_t;
# 76 "/usr/include/stdint.h" 3 4
typedef long int intptr_t;


typedef unsigned long int uintptr_t;
# 90 "/usr/include/stdint.h" 3 4
typedef __intmax_t intmax_t;
typedef __uintmax_t uintmax_t;
# 53 "/usr/bin/../lib/clang/18/include/stdint.h" 2 3
# 6 "../../c/uwapi-c/uwapi/common.h" 2

typedef uint8_t uint8;
typedef int8_t sint8;
typedef uint16_t uint16;
typedef int16_t sint16;
typedef uint32_t uint32;
typedef int32_t sint32;
typedef uint64_t uint64;
typedef int64_t sint64;
# 25 "../../c/uwapi-c/uwapi/common.h"
 static const uint32 UW_GameTicksPerSecond = 20;

 typedef struct UwIds
 {
  const uint32 *ids;
  uint32 count;
 } UwIds;



 typedef enum UwPrototypeTypeEnum
 {
  UwPrototypeTypeEnum_None = 0,
  UwPrototypeTypeEnum_Resource = 1,
  UwPrototypeTypeEnum_Recipe = 2,
  UwPrototypeTypeEnum_Construction = 3,
  UwPrototypeTypeEnum_Unit = 4,
 } UwPrototypeTypeEnum;
               void uwAllPrototypes(UwIds *data);
               UwPrototypeTypeEnum uwPrototypeType(uint32 prototypeId);
               const char *uwPrototypeJson(uint32 prototypeId);
               const char *uwDefinitionsJson(void);



 typedef struct UwMapInfo
 {
  const char *name;
  const char *guid;
  const char *path;
  uint32 maxPlayers;
 } UwMapInfo;
               _Bool uwMapInfo(UwMapInfo *data);



               uint32 uwTilesCount(void);
 typedef struct UwTile
 {
  float position[3];
  float up[3];
  const uint32 *neighborsIndices;
  uint32 neighborsCount;
  uint8 terrain;
  _Bool border;
 } UwTile;
               void uwTile(uint32 index, UwTile *data);



 typedef enum UwOverviewFlags
 {
  UwOverviewFlags_None = 0,
  UwOverviewFlags_Resource = 1 << 0,
  UwOverviewFlags_Construction = 1 << 1,
  UwOverviewFlags_MobileUnit = 1 << 2,
  UwOverviewFlags_StaticUnit = 1 << 3,
  UwOverviewFlags_Unit = UwOverviewFlags_MobileUnit | UwOverviewFlags_StaticUnit,
 } UwOverviewFlags;
               UwOverviewFlags uwOverviewFlags(uint32 position);
               void uwOverviewIds(uint32 position, UwIds *data);
 typedef struct UwOverviewExtract
 {
  const UwOverviewFlags *flags;
  uint32 count;
 } UwOverviewExtract;
               void uwOverviewExtract(UwOverviewExtract *data);



               void uwAreaRange(float x, float y, float z, float radius, UwIds *data);
               void uwAreaConnected(uint32 position, float radius, UwIds *data);
               void uwAreaNeighborhood(uint32 position, float radius, UwIds *data);
               void uwAreaExtended(uint32 position, float radius, UwIds *data);

               _Bool uwTestVisible(float x1, float y1, float z1, float x2, float y2, float z2);
               _Bool uwTestShooting(uint32 shooterPosition, uint32 shooterProto, uint32 targetPosition, uint32 targetProto);
               float uwDistanceLine(float x1, float y1, float z1, float x2, float y2, float z2);
               float uwDistanceEstimate(uint32 a, uint32 b);
               float uwYaw(uint32 position, uint32 towards);



 typedef struct UwEntity UwEntity;
               UwEntity *uwEntityPointer(uint32 id);
               uint32 uwEntityId(UwEntity *entity);
               void uwAllEntities(UwIds *data);



 typedef struct UwProtoComponent
 {
  uint32 proto;
 } UwProtoComponent;
               _Bool uwFetchProtoComponent(UwEntity *entity, UwProtoComponent *data);

 typedef struct UwOwnerComponent
 {
  uint32 force;
 } UwOwnerComponent;
               _Bool uwFetchOwnerComponent(UwEntity *entity, UwOwnerComponent *data);

 typedef struct UwControllerComponent
 {
  uint32 player;
  uint32 timestamp;
 } UwControllerComponent;
               _Bool uwFetchControllerComponent(UwEntity *entity, UwControllerComponent *data);

 typedef struct UwPositionComponent
 {
  uint32 position;
  float yaw;
 } UwPositionComponent;
               _Bool uwFetchPositionComponent(UwEntity *entity, UwPositionComponent *data);

 typedef enum UwUnitStateFlags
 {
  UwUnitStateFlags_None = 0,
  UwUnitStateFlags_Shooting = 1 << 0,
  UwUnitStateFlags_Processing = 1 << 1,
  UwUnitStateFlags_Rebuilding = 1 << 2,
 } UwUnitStateFlags;
 typedef struct UwUnitComponent
 {
  UwUnitStateFlags state;
  uint32 killCount;
 } UwUnitComponent;
               _Bool uwFetchUnitComponent(UwEntity *entity, UwUnitComponent *data);

 typedef struct UwLifeComponent
 {
  sint32 life;
 } UwLifeComponent;
               _Bool uwFetchLifeComponent(UwEntity *entity, UwLifeComponent *data);

 typedef struct UwMoveComponent
 {
  uint32 posStart;
  uint32 posEnd;
  uint32 tickStart;
  uint32 tickEnd;
  float yawStart;
  float yawEnd;
 } UwMoveComponent;
               _Bool uwFetchMoveComponent(UwEntity *entity, UwMoveComponent *data);

 typedef struct UwAimComponent
 {
  uint32 target;
 } UwAimComponent;
               _Bool uwFetchAimComponent(UwEntity *entity, UwAimComponent *data);

 typedef struct UwRecipeComponent
 {
  uint32 recipe;
 } UwRecipeComponent;
               _Bool uwFetchRecipeComponent(UwEntity *entity, UwRecipeComponent *data);

 typedef struct UwUpdateTimestampComponent
 {
  uint32 timestamp;
 } UwUpdateTimestampComponent;
               _Bool uwFetchUpdateTimestampComponent(UwEntity *entity, UwUpdateTimestampComponent *data);

 typedef struct UwRecipeStatisticsComponent
 {
  uint32 timestamps[3];
  uint32 completed;
 } UwRecipeStatisticsComponent;
               _Bool uwFetchRecipeStatisticsComponent(UwEntity *entity, UwRecipeStatisticsComponent *data);

 typedef enum UwPriorityEnum
 {
  UwPriorityEnum_Disabled = 0,
  UwPriorityEnum_Normal = 1,
  UwPriorityEnum_High = 2,
 } UwPriorityEnum;
 typedef struct UwPriorityComponent
 {
  UwPriorityEnum priority;
 } UwPriorityComponent;
               _Bool uwFetchPriorityComponent(UwEntity *entity, UwPriorityComponent *data);

 typedef struct UwAmountComponent
 {
  uint32 amount;
 } UwAmountComponent;
               _Bool uwFetchAmountComponent(UwEntity *entity, UwAmountComponent *data);

 typedef struct UwAttachmentComponent
 {
  uint32 target;
 } UwAttachmentComponent;
               _Bool uwFetchAttachmentComponent(UwEntity *entity, UwAttachmentComponent *data);

 typedef enum UwPlayerStateFlags
 {
  UwPlayerStateFlags_None = 0,
  UwPlayerStateFlags_Loaded = 1 << 0,
  UwPlayerStateFlags_Pause = 1 << 1,
  UwPlayerStateFlags_Disconnected = 1 << 2,
  UwPlayerStateFlags_Admin = 1 << 3,
 } UwPlayerStateFlags;
 typedef enum UwPlayerConnectionClassEnum
 {
  UwPlayerConnectionClassEnum_None = 0,
  UwPlayerConnectionClassEnum_Computer = 1,
  UwPlayerConnectionClassEnum_VirtualReality = 2,
  UwPlayerConnectionClassEnum_Robot = 3,
  UwPlayerConnectionClassEnum_UwApi = 4,
 } UwPlayerConnectionClassEnum;
 typedef struct UwPlayerComponent
 {
  char name[28];
  uint32 nameLength;
  uint64 steamUserId;
  uint32 force;
  float progress;
  uint32 ping;
  UwPlayerStateFlags state;
  UwPlayerConnectionClassEnum playerConnectionClass;
 } UwPlayerComponent;
               _Bool uwFetchPlayerComponent(UwEntity *entity, UwPlayerComponent *data);

 typedef enum UwForceStateFlags
 {
  UwForceStateFlags_None = 0,
  UwForceStateFlags_Winner = 1 << 0,
  UwForceStateFlags_Defeated = 1 << 1,
  UwForceStateFlags_Disconnected = 1 << 2,
 } UwForceStateFlags;
 typedef struct UwForceComponent
 {
  float color[3];
  uint64 score;
  uint32 killCount;
  uint32 lossCount;
  uint32 finishTimestamp;
  uint32 team;
  UwForceStateFlags state;
 } UwForceComponent;
               _Bool uwFetchForceComponent(UwEntity *entity, UwForceComponent *data);

 typedef struct UwForceDetailsComponent
 {
  uint64 killValue;
  uint64 lossValue;
  uint32 startingPosition;
 } UwForceDetailsComponent;
               _Bool uwFetchForceDetailsComponent(UwEntity *entity, UwForceDetailsComponent *data);

 typedef enum UwForeignPolicyEnum
 {
  UwForeignPolicyEnum_None = 0,
  UwForeignPolicyEnum_Self = 1,
  UwForeignPolicyEnum_Ally = 2,
  UwForeignPolicyEnum_Neutral = 3,
  UwForeignPolicyEnum_Enemy = 4,
 } UwForeignPolicyEnum;
 typedef struct UwForeignPolicyComponent
 {
  uint32 forces[2];
  UwForeignPolicyEnum policy;
 } UwForeignPolicyComponent;
               _Bool uwFetchForeignPolicyComponent(UwEntity *entity, UwForeignPolicyComponent *data);

 typedef struct UwDiplomacyProposalComponent
 {
  uint32 offeror;
  uint32 offeree;
  UwForeignPolicyEnum proposal;
 } UwDiplomacyProposalComponent;
               _Bool uwFetchDiplomacyProposalComponent(UwEntity *entity, UwDiplomacyProposalComponent *data);
# 5 "../../c/uwapi-c/uwapi/bots.h" 2








 static const uint32 UW_VERSION = 21;
               void uwInitialize(uint32 version);
               void uwDeinitialize(void);

 typedef void (*UwExceptionCallbackType)(const char *message);
               void uwSetExceptionCallback(UwExceptionCallbackType callback);

 typedef enum UwSeverityEnum
 {
  UwSeverityEnum_Note = 0,
  UwSeverityEnum_Hint = 1,
  UwSeverityEnum_Warning = 2,
  UwSeverityEnum_Info = 3,
  UwSeverityEnum_Error = 4,
  UwSeverityEnum_Critical = 5,
 } UwSeverityEnum;
 typedef struct UwLogCallback
 {
  const char *message;
  const char *component;
  UwSeverityEnum severity;
 } UwLogCallback;
 typedef void (*UwLogCallbackType)(const UwLogCallback *data);
               void uwSetLogCallback(UwLogCallbackType callback);
               void uwLog(UwSeverityEnum severity, const char *message);

 typedef struct UwAssistConfig
 {
  _Bool logistics;
  _Bool aiming;
  _Bool fighting;
  _Bool retaliations;
 } UwAssistConfig;
               void uwSetAssistConfig(const UwAssistConfig *config);

               void uwSetPlayerName(const char *name);
               void uwSetPlayerColor(float r, float g, float b);
               void uwSetConnectStartGui(_Bool enabled, const char *extraCmdParams);
               void uwSetConnectAsObserver(_Bool observer);

               _Bool uwConnectFindLan(uint64 timeoutMicroseconds);
               void uwConnectDirect(const char *address, uint16 port);
               void uwConnectLobbyId(uint64 lobbyId);
               void uwConnectNewServer(uint32 visibility, const char *name, const char *extraCmdParams);
               _Bool uwTryReconnect(void);
               void uwDisconnect(void);



 typedef enum UwConnectionStateEnum
 {
  UwConnectionStateEnum_None = 0,
  UwConnectionStateEnum_Connecting = 1,
  UwConnectionStateEnum_Connected = 2,
  UwConnectionStateEnum_Disconnecting = 3,
  UwConnectionStateEnum_Error = 4,
 } UwConnectionStateEnum;
 typedef void (*UwConnectionStateCallbackType)(UwConnectionStateEnum state);
               void uwSetConnectionStateCallback(UwConnectionStateCallbackType callback);
               UwConnectionStateEnum uwConnectionState(void);

 typedef enum UwGameStateEnum
 {
  UwGameStateEnum_None = 0,
  UwGameStateEnum_Session = 1,
  UwGameStateEnum_Preparation = 2,
  UwGameStateEnum_Game = 3,
  UwGameStateEnum_Finish = 4,
 } UwGameStateEnum;
 typedef void (*UwGameStateCallbackType)(UwGameStateEnum state);
               void uwSetGameStateCallback(UwGameStateCallbackType callback);
               UwGameStateEnum uwGameState(void);

 typedef enum UwMapStateEnum
 {
  UwMapStateEnum_None = 0,
  UwMapStateEnum_Downloading = 1,
  UwMapStateEnum_Loading = 2,
  UwMapStateEnum_Loaded = 3,
  UwMapStateEnum_Unloading = 4,
  UwMapStateEnum_Error = 5,
 } UwMapStateEnum;
 typedef void (*UwMapStateCallbackType)(UwMapStateEnum state);
               void uwSetMapStateCallback(UwMapStateCallbackType callback);
               UwMapStateEnum uwMapState(void);

 typedef void (*UwUpdateCallbackType)(uint32 tick, _Bool stepping);
               void uwSetUpdateCallback(UwUpdateCallbackType callback);



 typedef struct UwShootingUnit
 {
  uint32 position;
  uint32 force;
  uint32 prototype;
  uint32 id;
 } UwShootingUnit;
 typedef struct UwShootingData
 {
  UwShootingUnit shooter;
  UwShootingUnit target;
 } UwShootingData;
 typedef struct UwShootingArray
 {
  const UwShootingData *data;
  uint32 count;
 } UwShootingArray;
 typedef void (*UwShootingCallbackType)(const UwShootingArray *data);
               void uwSetShootingCallback(UwShootingCallbackType callback);



 typedef struct UwMyPlayer
 {
  uint32 playerEntityId;
  uint32 forceEntityId;
  _Bool primaryController;
  _Bool admin;
 } UwMyPlayer;
               _Bool uwMyPlayer(UwMyPlayer *data);



               void uwModifiedEntities(UwIds *data);



               _Bool uwTestConstructionPlacement(uint32 constructionPrototype, uint32 position);
               uint32 uwFindConstructionPlacement(uint32 constructionPrototype, uint32 position);



 typedef enum UwOrderTypeEnum
 {
  UwOrderTypeEnum_None = 0,
  UwOrderTypeEnum_Stop = 1,
  UwOrderTypeEnum_Guard = 2,
  UwOrderTypeEnum_Run = 3,
  UwOrderTypeEnum_Fight = 4,
  UwOrderTypeEnum_Load = 5,
  UwOrderTypeEnum_Unload = 6,
  UwOrderTypeEnum_SelfDestruct = 7,
 } UwOrderTypeEnum;
 typedef enum UwOrderPriorityFlags
 {
  UwOrderPriorityFlags_None = 0,
  UwOrderPriorityFlags_Assistant = 1 << 0,
  UwOrderPriorityFlags_User = 1 << 1,
  UwOrderPriorityFlags_Enqueue = 1 << 2,
  UwOrderPriorityFlags_Repeat = 1 << 3,
 } UwOrderPriorityFlags;
 typedef struct UwOrder
 {
  uint32 entity;
  uint32 position;
  UwOrderTypeEnum order;
  UwOrderPriorityFlags priority;
 } UwOrder;
               void uwOrder(uint32 unit, const UwOrder *data);
 typedef struct UwOrders
 {
  const UwOrder *orders;
  uint32 count;
 } UwOrders;
               void uwOrders(uint32 unit, UwOrders *data);



               void uwCommandPlaceConstruction(uint32 proto, uint32 position, float yaw);
               void uwCommandSetRecipe(uint32 unit, uint32 recipe);
               void uwCommandSetPriority(uint32 unit, UwPriorityEnum priority);
               void uwCommandLoad(uint32 unit, uint32 resourceType);
               void uwCommandUnload(uint32 unit);
               void uwCommandMove(uint32 unit, uint32 position, float yaw);
               void uwCommandAim(uint32 unit, uint32 target);
               void uwCommandRenounceControl(uint32 unit);
               void uwCommandSelfDestruct(uint32 unit);



               uint64 uwGetLobbyId(void);
               uint64 uwGetUserId(void);
               void uwAdminKickPlayer(uint32 player);
               void uwAdminPlayerSetAdmin(uint32 player, _Bool admin);
               void uwAdminPlayerSetName(uint32 player, const char *name);
               void uwAdminPlayerJoinForce(uint32 player, uint32 force);
               void uwAdminForceJoinTeam(uint32 force, uint32 team);
               void uwAdminForceSetColor(uint32 force, float r, float g, float b);
               void uwAdminAddAi(void);
               void uwAdminStartGame(void);
               void uwAdminTerminateGame(void);
               void uwAdminSetGameSpeed(float speed);
               void uwAdminSetWeatherSpeed(float speed, float offset);
               void uwSendCameraSuggestion(uint32 position);
               void uwSendMapSelection(const char *path);
